c     Copyright (C) 2009 Vincent Eymet
c
c     "satellite" is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     This software is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with the present software; if not, see <http://www.gnu.org/licenses/>
c
      subroutine intersected_sectors(dim,Ntheta,Nphi,theta,phi,
     &     Radius,rspace,Psat,Pground,op_alpha,
     &     Nint_sectors,int_sectors)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to identify which angular sectors are intersected by the observation cone
c
c     Inputs:
c       + dim: dimension of vectors
c       + Ntheta,Nphi: number of angular sectors over the latitude / over the longitude
c       + theta: latitude angular discretization
c       + phi: longitude angular discretization
c       + Radius: planet radius in meters
c       + rspace: radius at space level in meters
c       + Psat: altitude/latitude/longitude of satellite
c       + Pground: altitude/latitude/longitude of the point (at ground level)
c         intersected by the observation direction
c       + op_alpha: angle of the observation cone
c
c     Outputs:
c       + Nint_sectors: number of angular sectors that are intersected
c       + int_sectors: angular sectors that are intersected at space level (even partially)
c         by the observation cone
c         - int_sectors(i,1): itheta for sector index i
c         - int_sectors(i,2): iphi for sector index i
c         - 3 and 4: u_itheta and u_iphi in which the satellite is for sector index i
c

c     input
      integer dim,Nphi,Ntheta
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision Radius,rspace
      double precision Psat(1:Nmat_mx)
      double precision Pground(1:Nmat_mx)
      double precision op_alpha
c     output
      integer Nint_sectors
      integer int_sectors(1:Ntheta_mx*Nphi_mx,1:4)
c     temp
      integer intf,i,j
      integer s,Nsweeps
      double precision sat_r
      double precision u_obs(1:Nmat_mx)
      double precision Pint(1:Nmat_mx)
      double precision lphi_min,lphi_max
      double precision lphi
      integer itheta,iphi,u_itheta,u_iphi
      integer strlen
      character*100 label
      label='subroutine intersected_sectors'

      write(*,*) 'Identification of angular sectors...'

c     compute observation direction
      call observation_direction(dim,Psat,Pground,u_obs)
c     detect intersection between the line of sight and the ground
      call line_surface_r_intersect(0,dim,Psat,u_obs,
     &     Radius,-pi/2.0D+0,pi/2.0D+0,0.0D+0,2.0D+0*pi,intf,Pint)
      if (intf.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'Line of sight is not suitably defined'
         stop
      endif
c     sweeping the observation cone
      lphi_min=0.0D+0
      lphi_max=pi
c     Debug
      Nsweeps=100 ! number of sweep planes to be tested
c      Nsweeps=1 ! number of sweep planes to be tested
c     Debug
      Nint_sectors=0
      
      do s=1,Nsweeps
         lphi=lphi_min+(lphi_max-lphi_min)*
     &        dble(s-1)/dble(Nsweeps) ! rad
         call sweep(dim,Ntheta,Nphi,theta,phi,Psat,u_obs,
     &        rspace,op_alpha,lphi,
     &        Nint_sectors,int_sectors)
      enddo ! s
      if (Nint_sectors.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'Nint_sectors=',Nint_sectors
         stop
      endif

      write(*,*) '...done'

      return
      end



      subroutine sweep(dim,Ntheta,Nphi,theta,phi,Psat,u_obs,
     &     rspace,op_alpha,lphi,
     &     Nint_sectors,int_sectors)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to identify the angular sectors intersected by a given plane of observation
c     defined by angle "lphi"
c
c     Inputs:
c       + dim: dimension of vectors
c       + Ntheta,Nphi: number of angular sectors over the latitude / over the longitude
c       + theta: latitude angular discretization
c       + phi: longitude angular discretization
c       + Psat: altitude/latitude/longitude of satellite
c       + u_obs: normalized observation direction (cartesian coordinates)
c       + rspace: radius at space level in meters
c       + op_alpha: angle of the observation cone
c       + lphi: angle that defines the observation plane; this plane contains the observation
c         direction u_obs and the angle lphi (see comments in the code)
c
c     Outputs:
c       + Nint_sectors: number of angular sectors that are intersected
c       + int_sectors: angular sectors that are intersected at space level (even partially)
c     

c     input
      integer dim,Nphi,Ntheta
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision Psat(1:Nmat_mx)
      double precision u_obs(1:Nmat_mx)
      double precision rspace
      double precision op_alpha
      double precision lphi
c     output
      integer Nint_sectors
      integer int_sectors(1:Ntheta_mx*Nphi_mx,1:4)
c     temp
      integer Np,i,intf,cf,j
      integer itheta,iphi
      double precision ralpha,alpha
      double precision alpha_min,alpha_max
      double precision raf,theta_obs,phi_obs
      double precision j1(1:Nmat_mx)
      double precision axe(1:Nmat_mx)
      double precision u0(1:Nmat_mx)
      double precision u_lim(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision Pint(1:Nmat_mx)
      integer strlen
      character*100 label
      label='subroutine sweep'

c     Debug
      Np=1000 ! number of points along the observation plane
c      Np=2 ! number of points along the observation plane
c     Debug

c     ---------------------------------
c     computation of direction "u_lim"

c     theta_obs and phi_obs define the observation direction
      call cart2spher(u_obs(1),u_obs(2),u_obs(3),raf,theta_obs,phi_obs)
c     j1 is the direction perpendicular to u_obs, in a horizontal plane
      j1(1)=-dsin(phi_obs)
      j1(2)=dcos(phi_obs)
      j1(3)=0.0D+0
c     u0 is the direction defined by the intersection of the observation cone surface
c     and the vertical plane that contains u_obs
      call rotation(dim,u_obs,op_alpha,j1,u0)
c     u_lim is the transform of direction u0 after a rotation of angle lphi around
c     the observation direction u_obs
      call rotation(dim,u0,lphi,u_obs,u_lim)
c     axe is the local rotation angle (axe j1 rotated by lphi around u_obs)
      call rotation(dim,j1,lphi,u_obs,axe)
c     ---------------------------------

c     ---------------------------------
c     detection of limit angles on the sweeping plane
      alpha=0.0D+0
      call rotation(dim,u_lim,alpha,axe,u)
      call line_surface_r_intersect(0,dim,Psat,u,
     &     rspace,-pi/2.0D+0,pi/2.0D+0,0.0D+0,2.0D+0*pi,intf,Pint)
      if (intf.eq.1) then
         alpha_max=alpha
      else
         call find_alpha_lim(dim,u_lim,axe,Psat,rspace,
     &        0.0D+0,-op_alpha,
     &        alpha_max)
      endif
      alpha=-2.0D+0*op_alpha
      call rotation(dim,u_lim,alpha,axe,u)
      call line_surface_r_intersect(0,dim,Psat,u,
     &     rspace,-pi/2.0D+0,pi/2.0D+0,0.0D+0,2.0D+0*pi,intf,Pint)
      if (intf.eq.1) then
         alpha_min=alpha
      else
         call find_alpha_lim(dim,u_lim,axe,Psat,rspace,
     &        -op_alpha,-2.0D+0*op_alpha,
     &        alpha_min)
      endif
c     ---------------------------------

c     ---------------------------------
c     sweeping the observation plane

      do i=1,Np
c     ralpha is the rotation angle that defines the temporary line of sight
         ralpha=-(alpha_max-alpha_min)*dble(i-1)/dble(Np-1)
c     u is the temporary line of sight
         call rotation(dim,u_lim,ralpha,axe,u)
c     detect intersection between the line of sight and the sphere at space level
         call line_surface_r_intersect(0,dim,Psat,u,
     &        rspace,-pi/2.0D+0,pi/2.0D+0,0.0D+0,2.0D+0*pi,intf,Pint)
c     Debug
c         write(*,*) 'Pint=',(Pint(j),j=1,dim)
c     Debug
         if (intf.eq.1) then
c     when an intersection has been detected, identify the angular sector
            call identify_column(Pint,Ntheta,Nphi,theta,phi,
     &           cf,itheta,iphi)
            if (cf.eq.0) then
               write(*,*) 'Error from ',label(1:strlen(label)),' :'
               write(*,*) 'angular sector could not be identified'
               stop
            endif
            call add_sector(dim,Ntheta,Nphi,theta,phi,
     &           itheta,iphi,Psat,Pint,
     &           Nint_sectors,int_sectors)
         endif
      enddo ! i
c     ---------------------------------
      
      return
      end


      
      subroutine find_alpha_lim(dim,u_lim,axe,Psat,rspace,amin,amax,
     &     alpha_lim)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to identify the limit angle the planet is seen from in the given angular interval,
c     in the current sweeping plane
c
c     Inputs:
c       + dim: dimension of vectors
c       + u_lim: starting direction for sweeping
c       + axe: rotation angle for sweeping
c       + Psat: satellite position
c       + rspace: radius at space level
c       + (amin,amax): angular interval the limit angle has to be searched into
c
c     Outputs:
c       + alpha_lim: limit angle in the (amin,amax) interval
c     

c     input
      integer dim
      double precision u_lim(1:Nmat_mx)
      double precision axe(1:Nmat_mx)
      double precision Psat(1:Nmat_mx)
      double precision rspace
      double precision amin,amax
c     output
      double precision alpha_lim
c     temp
      integer Niter_max,iter,intf
      double precision a,a1,a2,da,da_min
      double precision u(1:Nmat_mx)
      double precision Pint(1:Nmat_mx)
      integer strlen
      character*100 label
      label='subroutine find_alpha_lim'
      Niter_max=1000
c     da_min is the angle variation (between two consecutive iterations)
c     below which convergence is accepted
      da_min=1.0D-10 ! rad

c     Debug
c      call rotation(dim,u_lim,amin,axe,u)
c      call line_surface_r_intersect(0,dim,Psat,u,
c     &     rspace,-pi/2.0D+0,pi/2.0D+0,0.0D+0,2.0D+0*pi,intf,Pint)
c      write(*,*) 'amin=',amin,' intf=',intf
c      call rotation(dim,u_lim,amax,axe,u)
c      call line_surface_r_intersect(0,dim,Psat,u,
c     &     rspace,-pi/2.0D+0,pi/2.0D+0,0.0D+0,2.0D+0*pi,intf,Pint)
c      write(*,*) 'amax=',amax,' intf=',intf
c     Debug


      a1=amin
      a2=amax
      iter=0
      da=dabs(a2-a1)
c     dichotomy procedure
      do while (da.gt.da_min)
         iter=iter+1
         if (iter.gt.Niter_max) then
            write(*,*) 'Error from ',label(1:strlen(label)),' :'
            write(*,*) 'maximum number of iterations reached'
            stop
         endif
         a=(a1+a2)/2.0D+0
         call rotation(dim,u_lim,a,axe,u)
         call line_surface_r_intersect(0,dim,Psat,u,
     &        rspace,-pi/2.0D+0,pi/2.0D+0,0.0D+0,2.0D+0*pi,intf,Pint)
         if (intf.eq.0) then
            a1=a
         else
            a2=a
         endif
         da=dabs(a2-a1)
      enddo
 123  continue
      alpha_lim=(a1+a2)/2.0D+0

      return
      end




      subroutine add_sector(dim,Ntheta,Nphi,theta,phi,
     &     itheta,iphi,Psat,Pint,
     &     Nint_sectors,int_sectors)
      implicit none
      include 'max.inc'
c
c     Purpose: to add the current angular sector defined by (itheta,iphi) to the
c     array of angular sectors "int_sectors", if the current angular sector
c     does not already belong to this array.
c
c     Inputs:
c       + dim: dimension of vectors
c       + Ntheta,Nphi: number of angular sectors over the latitude / over the longitude
c       + theta: latitude angular discretization
c       + phi: longitude angular discretization
c       + (itheta,iphi): indexes that defines the current angular sector
c       + Psat: satellite position
c       + Pint: position intersected by the current line of sight
c       + Nint_sectors: number of angular sectors that are intersected
c       + int_sectors: angular sectors that are intersected at space level (even partially)
c     
c     Outputs: updated "Nint_sectors" and "int_sectors" if relevant
c     

c     I/O
      integer dim,Nphi,Ntheta
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      integer itheta,iphi
      double precision Psat(1:Nmat_mx)
      double precision Pint(1:Nmat_mx)
      integer Nint_sectors
      integer int_sectors(1:Ntheta_mx*Nphi_mx,1:4)
c     temp
      integer i,sf
      integer u_itheta,u_iphi
      double precision u(1:Nmat_mx)
      integer strlen
      character*100 label
      label='subroutine add_sector'

      sf=0
      do i=1,Nint_sectors
         if ((int_sectors(i,1).eq.itheta).and.
     &        (int_sectors(i,2).eq.iphi)) then
            sf=1
            goto 123
         endif
      enddo ! i
 123  continue
      if (sf.eq.0) then ! only in the case the current angular sector has not been identified
         Nint_sectors=Nint_sectors+1
         if (Nint_sectors.gt.Ntheta_mx*Nphi_mx) then
            write(*,*) 'Error from ',label(1:strlen(label)),' :'
            write(*,*) 'max. number of ang. sectors has been reached'
            stop
         endif
         int_sectors(Nint_sectors,1)=itheta
         int_sectors(Nint_sectors,2)=iphi
c     modify Pint so that it is in the middle of the (itheta,iphi) angular sector
         Pint(2)=(theta(itheta-1)+theta(itheta))/2.0D+0
         Pint(3)=(phi(iphi-1)+phi(iphi))/2.0D+0
c     identification of the sector the satellite is seen in from Pint
         call observation_direction(dim,Pint,Psat,u)
         call sort_direction(dim,Ntheta,Nphi,theta,phi,u,
     &        u_itheta,u_iphi)
         int_sectors(Nint_sectors,3)=u_itheta
         int_sectors(Nint_sectors,4)=u_iphi
      endif

      return
      end



      subroutine observation_direction(dim,Psat,Pint,u_obs)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the observation direction from basic geometric data
c     
c     Inputs:
c       + dim: dimension of vectors
c       + Psat: altitude/latitude/longitude of satellite
c       + Pint: altitude/latitude/longitude of the position that
c         defines the observation direction
c
c     Outputs:
c       + u_obs: normalized observation direction (cartesian coordinates)
c

c     inputs
      integer dim
      double precision Psat(1:Nmat_mx)
      double precision Pint(1:Nmat_mx)
c     outputs
      double precision u_obs(1:Nmat_mx)
c     temp
      double precision sat(1:Nmat_mx)
      double precision point(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      integer strlen
      character*100 label
      label='subroutine observation_direction'
      
c     convert spherical coordinates to cartesian coordinates
      call spher2cart(label,1,Psat(1),Psat(2),Psat(3),
     &     sat(1),sat(2),sat(3))
      call spher2cart(label,1,Pint(1),Pint(2),Pint(3),
     &     point(1),point(2),point(3))
c     substract positions -> direction vector
      call substract_vectors(dim,point,sat,u)
c     normalization
      call normalize_vector(dim,u,u_obs)

      return
      end
