c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine read_grid(Radius,Nr,Ntheta,Nphi,Nb_gas,Nq,Nzones,
     &     z,r,theta,phi,rground,rspace,lambda_gas_lo,lambda_gas_hi,
     &     w_quad,ka_gas,ks_gas,r_mc,Tground,T,Tspace,
     &     ground_emissivity,space_emissivity,
     &     emissivity_zone,emissivity,n_ref)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to read information about the grid
c     requires: file "data/grid.in"
c
c     input:
c       + Radius: radius of the planetary reference level
c
c     output:
c       + Nr: number of cells along the vertical
c       + Ntheta: number of cells along the latitude
c       + Nphi: number of cells along the longitude
c       + Nb_gas: number of narrowband spectral intervals for gas
c       + Nq: quadrature order
c       + Nzones: number of special emissivity zones
c       + z(ir), ir=0,Nr: altitude (m) of each vertical interface
c       + r(ir), ir=0,Nr: radius (m) of each vertical interface
c       + theta(itheta), itheta=0,Ntheta: latitude (rad) of each latitude interface
c       + phi(iphi), iphi=0,Nphi: longitude (rad) of each longitude interface
c       + rground: radius at ground level
c       + rspace: radius at space level
c       + lambda_gas_lo: lower limits of narrowband spectral intervals (m)
c       + lambda_gas_hi: higher limits of narrowband spectral intervals (m)
c       + w_quad: quadrature weights
c       + ka_gas: gas absorption coefficient (m)
c       + ks_gas: gas scattering coefficient (m)
c       + r_mc(ir,itheta,iphi): radius at the mass center for each cell (ir=1,Nr)
c       + Tground(itheta,iphi): temperature of the ground for a given column
c       + T(ir,itheta,iphi): temperature at mass center for each cell for ir=1,Nr
c                            temperature of the gas at ground level (ir=0) for a given column
c                            temperature of the gas at TOM (ir=Nr+1) for a given column
c       + Tspace(itheta,iphi): temperature of space for a given column
c       + ground_emissivity: emissivity of ground for column index (itheta,iphi)
c       + space_emissivity: emissivity of space for column index (itheta,iphi)
c       + emissivity_zone(izone,j): spatial dimenstions of special emissivity zones
c         - (izone,1): minimum latitude (rad) of the zone
c         - (izone,2): maximum latitude (rad) of the zone
c         - (izone,3): minimum longitude (rad) of the zone
c         - (izone,4): maximum longitude (rad) of the zone
c       + emissivity: values of emissivity for each special zone
c         - (izone,band): emissivity of the zone index 'izone' in narrowband index 'band'
c       + n_ref: refraction index for each cell, for each narrowband
c

c     input
      double precision Radius
c     output
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi,Nb_gas,Nq,Nzones
      double precision z(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision rground,rspace
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision lambda_gas_lo(1:Nb_gas_mx)
      double precision lambda_gas_hi(1:Nb_gas_mx)
      double precision w_quad(1:Nq_mx)
      double precision ka_gas(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx,1:Nq_mx)
      double precision ks_gas(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx,1:Nq_mx)
      double precision r_mc(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision Tground(1:Ntheta_mx,1:Nphi_mx)
      double precision T(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision Tspace(1:Ntheta_mx,1:Nphi_mx)
      double precision ground_emissivity(1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx)
      double precision space_emissivity(1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx)
      double precision emissivity_zone(1:Nzones_mx,1:4)
      double precision emissivity(1:Nzones_mx,1:Nb_gas_mx)
      double precision n_ref(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
c     temp
      integer ok
      integer ios,ir,itheta,iphi,izone,j,strlen,band,quad
      double precision z_mc(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision zground,zspace
      character*100 label,datafile
      label='subroutine read_grid'

      datafile='./data/grid.in'
      open(10,file=datafile(1:strlen(datafile)),
     &     status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'File could not be found:'
         write(*,*) datafile(1:strlen(datafile))
         stop
      endif

      read(10,*) Ntheta,Nphi,Nb_gas,Nq,Nzones
      if (Ntheta.gt.Ntheta_mx) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'Ntheta=',Ntheta
         write(*,*) 'is greater than Ntheta_mx=',Ntheta_mx
         stop
      endif
      if (Nphi.gt.Nphi_mx) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'Nphi=',Nphi
         write(*,*) 'is greater than Nphi_mx=',Nphi_mx
         stop
      endif
      if (Nb_gas.gt.Nb_gas_mx) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'Nb_gas=',Nb_gas
         write(*,*) 'is greater than Nb_gas_mx=',Nb_gas_mx
         stop
      endif
      if (Nq.gt.Nq_mx) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'Nq=',Nq
         write(*,*) 'is greater than Nq_mx=',Nq_mx
         stop
      endif
      if (Nzones.gt.Nzones_mx) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'Nzones=',Nzones
         write(*,*) 'is greater than Nzones_mx=',Nzones_mx
         stop
      endif
      do itheta=1,Ntheta
         do iphi=1,Nphi
            read(10,*) Nr(itheta,iphi)
            if (Nr(itheta,iphi).gt.Nz_mx) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),':'
               write(*,*) 'Nr(',itheta,',',iphi,')=',Nr(itheta,iphi)
               write(*,*) 'is greater than Nz_mx=',Nz_mx
               stop
            endif
            read(10,50) (z(ir,itheta,iphi),ir=0,Nr(itheta,iphi)) ! m
         enddo ! iphi
      enddo ! itheta

c      call check_ground_altitudes(Ntheta,Nphi,z,zground)

c     check that the altitude of space is the same for every column
      call check_space_altitudes(Nr,Ntheta,Nphi,z,zspace)

c     Debug
c     If you have to modify altitudes, do it here
c      z(0,10,1)=z(0,10,1)+1.0D+2 ! m
c      z(0,10,2)=z(0,10,2)-1.0D+1 ! m
c      z(0,10,3)=z(0,10,3)+3.0D+1 ! m
c      z(0,11,1)=z(0,11,1)+1.0D+1 ! m
c      z(0,11,2)=z(0,11,2)-2.0D+1 ! m
c      z(0,11,3)=z(0,11,3)-1.0D+1 ! m
c      z(0,12,1)=z(0,12,1)-2.5D+1 ! m
c      z(0,12,2)=z(0,12,2)-1.5D+1 ! m
c      z(0,12,3)=z(0,12,3)+2.5D+1 ! m
c      z(0,10,40)=z(0,10,40)+0.5D+1 ! m
c      z(0,11,40)=z(0,11,40)-2.5D+1 ! m
c      z(0,12,40)=z(0,12,40)-1.5D+1 ! m
c     Debug

c     computing radiuses from altitudes
      do itheta=1,Ntheta
         do iphi=1,Nphi
            do ir=0,Nr(itheta,iphi)
               call z2r(Radius,z(ir,itheta,iphi),r(ir,itheta,iphi))
            enddo ! ir
         enddo ! iphi
      enddo                     ! itheta
      call z2r(Radius,zground,rground)
      call z2r(Radius,zspace,rspace)


c     check that no radius at a given level is lower than the radius at a previous level
      call check_levels_radiuses(Nr,Ntheta,Nphi,r,ok)
      if (ok.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'levels radiuses do not meet dependencies'
         stop
      endif

c     check that the radius of ground at the center of each column is not above
c     the radius at the top of layer 1, for every adjacent column
      call check_rground_consistency(Nr,Ntheta,Nphi,r,ok)
      if (ok.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'ground radiuses do not meet dependencies'
         stop
      endif

      read(10,50) (theta(itheta),itheta=0,Ntheta) ! rad
      read(10,50) (phi(iphi),iphi=0,Nphi) ! rad
c     altitudes of mass center
      do itheta=1,Ntheta
         do iphi=1,Nphi
            read(10,50) (z_mc(ir,itheta,iphi),ir=1,Nr(itheta,iphi)) ! m
            do ir=1,Nr(itheta,iphi)
               call z2r(Radius,
     &              z_mc(ir,itheta,iphi),
     &              r_mc(ir,itheta,iphi))
            enddo ! ir
         enddo ! iphi
      enddo ! itheta
c     temperature profiles
      do itheta=1,Ntheta
         do iphi=1,Nphi
            read(10,50) Tground(itheta,iphi),
     &           (T(ir,itheta,iphi),ir=0,Nr(itheta,iphi)+1),
     &           Tspace(itheta,iphi)
         enddo ! iphi
      enddo ! itheta
c     spectral stuff
      read(10,50) (lambda_gas_lo(band),band=1,Nb_gas)
      read(10,50) (lambda_gas_hi(band),band=1,Nb_gas)
      read(10,50) (w_quad(quad),quad=1,Nq)
      do itheta=1,Ntheta
         do iphi=1,Nphi
            read(10,50) (ground_emissivity(itheta,iphi,band),
     &           band=1,Nb_gas)
            read(10,50) (space_emissivity(itheta,iphi,band),
     &           band=1,Nb_gas)
         enddo ! iphi
      enddo ! itheta
      do itheta=1,Ntheta
         do iphi=1,Nphi
            do ir=1,Nr(itheta,iphi)
               do band=1,Nb_gas
                  read(10,50) (ka_gas(ir,itheta,iphi,band,quad),
     &                 quad=1,Nq)
                  read(10,50) (ks_gas(ir,itheta,iphi,band,quad),
     &                 quad=1,Nq)
               enddo            ! band
            enddo               ! ir
         enddo                  ! iphi
      enddo                     ! itheta
c     special emissivity zones data
      do izone=1,Nzones
         read(10,50) (emissivity_zone(izone,j),j=1,4)
      enddo
      do izone=1,Nzones
         read(10,50) (emissivity(izone,band),band=1,Nb_gas)
      enddo
c     refraction index
      do itheta=1,Ntheta
         do iphi=1,Nphi
            do band=1,Nb_gas
               read(10,50) (n_ref(ir,itheta,iphi,band),
     &              ir=1,Nr(itheta,iphi))
            enddo ! band
         enddo ! iphi
      enddo ! itheta

      close(10)

      return
      end



      subroutine check_rground_consistency(Nr,Ntheta,Nphi,r,ok)
      implicit none
      include 'max.inc'
c     
c     Purpose: to check consistency between ground radiuses
c     The code will not work properly if any ground radius is above the radius at the top
c     of the first layer for adjacent columns
c
c     Inputs:
c       + Nr,Ntheta,Nphi: number of cells
c       + r: radiuses
c
c     Output:
c       + ok: 1 if radiuses satisfy dependances; 0 otherwise
c

c     inputs
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
c     output
      integer ok
c     temp
      integer ir,itheta,iphi
      integer itheta_prev,itheta_next
      integer iphi_prev,iphi_next,iphi_mod
      integer strlen
      character*100 label
      label='subroutine check_rground_consistency'

      ok=1
      do itheta=2,Ntheta-1
         do iphi=1,Nphi
            call prev_theta_cell(Ntheta,Nphi,itheta,iphi,
     &           itheta_prev,iphi_mod)
            call prev_phi_cell(Nphi,iphi,iphi_prev)
            call next_theta_cell(Ntheta,Nphi,itheta,iphi,
     &           itheta_next,iphi_mod)
            call next_phi_cell(Nphi,iphi,iphi_next)
c     itheta+1,iphi-1
            if (r(0,itheta,iphi).gt.r(1,itheta_next,iphi_prev)) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'r(0,',itheta,',',iphi,')=',r(0,itheta,iphi)
               write(*,*) 'is above r(1,',itheta_next,',',iphi_prev,
     &              ')=',r(1,itheta_next,iphi_prev)
               ok=0
               goto 666
            endif
c     itheta+1,iphi
            if (r(0,itheta,iphi).gt.r(1,itheta_next,iphi)) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'r(0,',itheta,',',iphi,')=',r(0,itheta,iphi)
               write(*,*) 'is above r(1,',itheta_next,',',iphi,
     &              ')=',r(1,itheta_next,iphi)
               ok=0
               goto 666
            endif
c     itheta+1,iphi+1
            if (r(0,itheta,iphi).gt.r(1,itheta_next,iphi_next)) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'r(0,',itheta,',',iphi,')=',r(0,itheta,iphi)
               write(*,*) 'is above r(1,',itheta_next,',',iphi_next,
     &              ')=',r(1,itheta_next,iphi_next)
               ok=0
               goto 666
            endif

c     itheta,iphi-1
            if (r(0,itheta,iphi).gt.r(1,itheta,iphi_prev)) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'r(0,',itheta,',',iphi,')=',r(0,itheta,iphi)
               write(*,*) 'is above r(1,',itheta,',',iphi_prev,
     &              ')=',r(1,itheta,iphi_prev)
               ok=0
               goto 666
            endif
c     itheta,iphi+1
            if (r(0,itheta,iphi).gt.r(1,itheta,iphi_next)) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'r(0,',itheta,',',iphi,')=',r(0,itheta,iphi)
               write(*,*) 'is above r(1,',itheta,',',iphi_next,
     &              ')=',r(1,itheta,iphi_next)
               ok=0
               goto 666
            endif

c     itheta-1,iphi-1
            if (r(0,itheta,iphi).gt.r(1,itheta_prev,iphi_prev)) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'r(0,',itheta,',',iphi,')=',r(0,itheta,iphi)
               write(*,*) 'is above r(1,',itheta_prev,',',iphi_prev,
     &              ')=',r(1,itheta_prev,iphi_prev)
               ok=0
               goto 666
            endif
c     itheta-1,iphi
            if (r(0,itheta,iphi).gt.r(1,itheta_prev,iphi)) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'r(0,',itheta,',',iphi,')=',r(0,itheta,iphi)
               write(*,*) 'is above r(1,',itheta_prev,',',iphi,
     &              ')=',r(1,itheta_prev,iphi)
               ok=0
               goto 666
            endif
c     itheta-1,iphi+1
            if (r(0,itheta,iphi).gt.r(1,itheta_prev,iphi_next)) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'r(0,',itheta,',',iphi,')=',r(0,itheta,iphi)
               write(*,*) 'is above r(1,',itheta_prev,',',iphi_next,
     &              ')=',r(1,itheta_prev,iphi_next)
               ok=0
               goto 666
            endif

         enddo ! iphi
      enddo ! itheta

c     south pole
      itheta=1
      do iphi=1,Nphi
         call prev_phi_cell(Nphi,iphi,iphi_prev)
         call next_phi_cell(Nphi,iphi,iphi_next)
         if (r(0,itheta,iphi).gt.r(1,itheta,iphi_prev)) then
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'r(0,',itheta,',',iphi,')=',r(0,itheta,iphi)
            write(*,*) 'is above r(1,',itheta,',',iphi_prev,
     &           ')=',r(1,itheta,iphi_prev)
            ok=0
            goto 666
         endif
         if (r(0,itheta,iphi).gt.r(1,itheta,iphi_next)) then
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'r(0,',itheta,',',iphi,')=',r(0,itheta,iphi)
            write(*,*) 'is above r(1,',itheta,',',iphi_next,
     &           ')=',r(1,itheta,iphi_next)
            ok=0
            goto 666
         endif
      enddo ! iphi

c     north pole
      itheta=Ntheta
      do iphi=1,Nphi
         call prev_phi_cell(Nphi,iphi,iphi_prev)
         call next_phi_cell(Nphi,iphi,iphi_next)
         if (r(0,itheta,iphi).gt.r(1,itheta,iphi_prev)) then
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'r(0,',itheta,',',iphi,')=',r(0,itheta,iphi)
            write(*,*) 'is above r(1,',itheta,',',iphi_prev,
     &           ')=',r(1,itheta,iphi_prev)
            ok=0
            goto 666
         endif
         if (r(0,itheta,iphi).gt.r(1,itheta,iphi_next)) then
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'r(0,',itheta,',',iphi,')=',r(0,itheta,iphi)
            write(*,*) 'is above r(1,',itheta,',',iphi_next,
     &           ')=',r(1,itheta,iphi_next)
            ok=0
            goto 666
         endif
      enddo ! iphi

 666  continue
      return
      end



      subroutine check_levels_radiuses(Nr,Ntheta,Nphi,r,ok)
      implicit none
      include 'max.inc'
c
c     Purpose: to check that the radius for every level is
c     greater than the radius at the previous level
c
c     Inputs:
c       + Nr,Ntheta,Nphi: number of cells
c       + r: values of radiuses
c     
c     Output:
c       + ok: 1 if requirements are met; 0 otherwise
c

c     input
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
c     output
      integer ok
c     temp
      integer ir,itheta,iphi
      integer strlen
      character*100 label
      label='subroutine check_levels_altitudes'

      ok=1
      do itheta=1,Ntheta
         do iphi=1,Nphi
            do ir=1,Nr(itheta,iphi)
               if (r(ir,itheta,iphi).lt.r(ir-1,itheta,iphi)) then
                  write(*,*) 'Error from ',
     &                 label(1:strlen(label)),' :'
                  write(*,*) 'r(',ir,',',itheta,',',iphi,')=',
     &                 r(ir,itheta,iphi)
                  write(*,*) 'is lower than:'
                  write(*,*) 'r(',ir-1,',',itheta,',',iphi,')=',
     &                 r(ir-1,itheta,iphi)
                  ok=0
                  goto 666
               endif
            enddo ! ir
         enddo ! iphi
      enddo ! itheta

 666  continue
      return
      end



      subroutine check_ground_altitudes(Ntheta,Nphi,z,zground)
      implicit none
      include 'max.inc'
c
c     Purpose: to check if ground altitudes are identical for all columns
c     NOTE: this routine is no longer used, since altitudes at ground
c     level may differ from one column to the other
c
c     Inputs:
c       + Ntheta, Nphi: number of cells
c       + z: altitudes
c
c     Output:
c       + zground: altitude at ground level
c

c     input
      integer Ntheta,Nphi
      double precision z(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
c     output
      double precision zground
c     temp
      double precision z0
      integer itheta,iphi
      integer strlen
      character*100 label
      label='subroutine check_ground_altitudes'

      z0=z(0,1,1)
      do itheta=1,Ntheta
         do iphi=1,Nphi
            if (z(0,itheta,iphi).ne.z0) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'Ground altitude is not coherent for column:'
               write(*,*) 'itheta=',itheta
               write(*,*) 'iphi=',iphi
               stop
            endif
         enddo ! iphi
      enddo ! itheta
      zground=z0

      return
      end



      subroutine check_space_altitudes(Nr,Ntheta,Nphi,z,zspace)
      implicit none
      include 'max.inc'
c
c     Purpose: to check if space altitudes are identical for all columns
c
c     Inputs:
c       + Ntheta, Nphi: number of cells
c       + z: altitudes
c
c     Output:
c       + zspace: (unique) altitude at space level
c

c     input
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      double precision z(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
c     output
      double precision zspace
c     temp
      double precision z0
      integer itheta,iphi
      integer strlen
      character*100 label
      label='subroutine check_space_altitudes'

      z0=z(Nr(1,1),1,1)
      do itheta=1,Ntheta
         do iphi=1,Nphi
            if (z(Nr(itheta,iphi),itheta,iphi).ne.z0) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'Space altitude is not coherent for column:'
               write(*,*) 'itheta=',itheta
               write(*,*) 'iphi=',iphi
               stop
            endif
         enddo ! iphi
      enddo ! itheta
      zspace=z0

      return
      end





      subroutine find_dr_min(Nr,Ntheta,Nphi,r,dr_min)
      implicit none
      include 'max.inc'
c
c     Purpose: to find the minimum layer thickness (in meters)
c
c     Inputs:
c       + Nr,Ntheta, Nphi: number of cells
c       + r: radiuses
c
c     Output:
c       + dr_min: minimum layer thickness (m)
c

c     input
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
c     output
      double precision dr_min
c     temp
      integer ir,itheta,iphi
      double precision dr
      integer strlen
      character*100 label
      label='subroutine find_dz_min'

      dr_min=r(Nr(1,1),1,1)-r(Nr(1,1)-1,1,1)
      do itheta=1,Ntheta
         do iphi=1,Nphi
            do ir=1,Nr(itheta,iphi)
               dr=r(ir,itheta,iphi)-r(ir-1,itheta,iphi)
               if (dr.lt.dr_min) then
                  dr_min=dr
               endif
            enddo ! ir
         enddo ! iphi
      enddo ! itheta

      return
      end

